#include "CMatrixRotate.h"

#include "Common.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CMatrixRotate

CMatrixRotate::CMatrixRotate(unsigned int SizeX, unsigned int SizeY)
	:m_SizeX(SizeX), m_SizeY(SizeY), m_hM(NULL), m_hMR(NULL), m_dM(NULL),
	m_dMR(NULL), m_hGPUResultNaive(NULL), m_hGPUResultOpt(NULL)
{
}

CMatrixRotate::~CMatrixRotate()
{
}

bool CMatrixRotate::InitResources(cl_device_id Device, cl_context Context)
{
	//CPU resources
	m_hM = new float[m_SizeX * m_SizeY];
	m_hMR = new float[m_SizeX * m_SizeY];
	m_hGPUResultNaive = new float[m_SizeX * m_SizeY];
	m_hGPUResultOpt = new float[m_SizeX * m_SizeY];

	//fill the matrix with random floats
	for(unsigned int i = 0; i < m_SizeX * m_SizeY; i++)
	{
		m_hM[i] = float(rand()) / float(RAND_MAX);
	}

	//device resources...

	return true;
}

void CMatrixRotate::ReleaseResources()
{
	//CPU resources
	SAFE_DELETE_ARRAY(m_hM);
	SAFE_DELETE_ARRAY(m_hMR);
	SAFE_DELETE_ARRAY(m_hGPUResultNaive);
	SAFE_DELETE_ARRAY(m_hGPUResultOpt);

	//device resources...
}

void CMatrixRotate::ComputeGPU(cl_context Context, cl_command_queue CommandQueue, size_t LocalWorkSize[3])
{
	//write input data to the GPU...

	//detemine the necessary number of global work items...

	///////////////////////////////////
	//naive kernel

	double time = 0;

	//run kernel using RunKernelNTimes()...

	cout<<"Executed naive kernel in "<<time<<" ms."<<endl;
	
	//read back the results synchronously.
	//this command has to be blocking, since we want to check the valid data

	///////////////////////////////////
	//optimized kernel
	
	//allocate shared (local) memory for the kernel...
	
	//run kernel...

	cout<<"Executed optimized kernel in "<<time<<" ms."<<endl;

	//read back data...
}

void CMatrixRotate::ComputeCPU()
{
	//Uncomment the code corresponding to the direction in which you want to rotate the matrix!

	//// Rotate clockwise
	//for(unsigned int x = 0; x < m_SizeX; x++)
	//{
	//	// MR(sY - y - 1, x) = M(x, y)
	//	for(unsigned int y = 0; y < m_SizeY; y++)
	//	{
	//		m_hMR[ x * m_SizeY + (m_SizeY - y - 1) ] = m_hM[ y * m_SizeX + x ];
	//	}
	//}

	//// Rotate counterclockwise
	//for(unsigned int x = 0; x < m_SizeX; x++)
	//{
	//	for(unsigned int y = 0; y < m_SizeY; y++)
	//	{
	//		// MR(y, Sx - x - 1) = M(x, y) 
	//		m_hMR[ (m_SizeX - x - 1) * m_SizeY + y ] = m_hM[ y * m_SizeX + x ];
	//	}
	//}
	///////////////////////////////////////////////////////////////////////

}

bool CMatrixRotate::ValidateResults()
{
	if(!memcmp(m_hMR, m_hGPUResultNaive, m_SizeX * m_SizeY * sizeof(float)) == 0)
	{
		cout<<"Results of the naive kernel are incorrect!"<<endl;
		return false;
	}
	if(!memcmp(m_hMR, m_hGPUResultOpt, m_SizeX * m_SizeY * sizeof(float)) == 0)
	{
		cout<<"Results of the optimized kernel are incorrect!"<<endl;
		return false;
	}
	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
