/*
This task demonstrates the benefits of using the shared memory
on a very simple example: given a large matrix of numbers, we would like to generate
a new matrix as a result of a clockwise rotation.

Compared to the naive implementation, the shared memory can be used as a temporary storage of
processed data to keep memory access patterns by threads efficient (coalesced).
*/

#ifndef CMATRIXROTATE_H
#define CMATRIXROTATE_H

#include "IAssignment.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CMatrixRotate
	:public IAssignment
{
protected:
	//NOTE: we have two memory address spaces, so we mark pointers with a prefix
	//to avoid confusions: 'h' - host, 'd' - device

	unsigned int		m_SizeX;
	unsigned int		m_SizeY;

	//float data on the CPU
	//M: original matrix, MR: rotated matrix
	float				*m_hM, *m_hMR;

	//pointers on the GPU
	//(result buffers for both kernels)
	cl_mem				m_dM, m_dMR;
	//(..and a pointer to read back the result)
	float				*m_hGPUResultNaive, *m_hGPUResultOpt;

	//Add OpenCL program and kernels!

public:
	CMatrixRotate(unsigned int SizeX, unsigned int SizeY);

	virtual ~CMatrixRotate();

	// IAssigment
	
	virtual bool InitResources(cl_device_id Device, cl_context Context);
	
	virtual void ReleaseResources();

	virtual void ComputeGPU(cl_context Context, cl_command_queue CommandQueue, size_t LocalWorkSize[3]);

	virtual void ComputeCPU();

	virtual bool ValidateResults();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif