/*
This task introduces the concepts of parallel programming on the GPU on the
following example:
You need to add two arrays of integers by elements, where one of the arrays
stores the elements in a reverse order. The "traditional" implementation on the CPU
is already included. Your task is to implement the same "algorithm" in an OpenCL kernel,
and produce identical results in a parallel way.
*/

#ifndef CSIMPLEARRAYS_H
#define CSIMPLEARRAYS_H

#include "IAssignment.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CSimpleArrays
	:public IAssignment
{
protected:

	//NOTE: we have two memory address spaces, so we mark pointers with a prefix
	//to avoid confusions: 'h' - host, 'd' - device
	
	//number of array elements
	unsigned int	m_ArraySize;

	//integer arrays on the CPU
	int			*m_hA, *m_hB, *m_hC;

	//integer arrays on the GPU (and a buffer to read the result back to the host)
	cl_mem			m_dA, m_dB, m_dC;
	int			*m_hGPUResult;

	/////////////////////////////////////////
	// Sect. 4.6.
	
	//Add OpenCL program and kernels!

public:
	CSimpleArrays(unsigned int ArraySize);
	virtual ~CSimpleArrays();

	// IAssigment
	
	virtual bool InitResources(cl_device_id Device, cl_context Context);
	
	virtual void ReleaseResources();

	virtual void ComputeGPU(cl_context Context, cl_command_queue CommandQueue, size_t LocalWorkSize[3]);

	virtual void ComputeCPU();

	virtual bool ValidateResults();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif
