// All OpenCL headers
#if defined (__APPLE__) || defined(MACOSX)
    #include <OpenCL/opencl.h>
#else
    #include <CL/opencl.h>
#endif 

#include "Common.h"
#include "CSimpleArrays.h"
#include "CMatrixRotate.h"

using namespace std;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global variables and forward declarations

cl_context			g_CLContext			= NULL;
cl_command_queue	g_CLCommandQueue	= NULL;
cl_platform_id		g_CLPlatform		= NULL;
cl_device_id		g_CLDevice			= NULL;

bool InitContextResources();
void CleanupContextResources();
bool RunAssigment(IAssignment& Assignment, size_t LocalWorkSize[3]);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char** argv)
{
	if(InitContextResources())
	{
		cout<<"OpenCL context initialized."<<endl;

		cout<<"Running vector addition example..."<<endl<<endl;
		//task 1: simple array addition
		{
			size_t LocalWorkSize[3] = {256, 1, 1};
			CSimpleArrays assignment(1048576);
			RunAssigment(assignment, LocalWorkSize);
		}
		{
			size_t LocalWorkSize[3] = {512, 1, 1};
			CSimpleArrays assignment(1048576);
			RunAssigment(assignment, LocalWorkSize);
		}

		//task 2: matrix rotation
		cout<<"Running matrix rotation example..."<<endl<<endl;
		{
			size_t LocalWorkSize[3] = {32, 16, 1};
			CMatrixRotate assignment(2048, 1024);
			RunAssigment(assignment, LocalWorkSize);
		}
	}

	CleanupContextResources();
	
	cout<<"Press any key..."<<endl;
	cin.get();
}

bool InitContextResources()
{
	//////////////////////////////////////////////////////
	//(Sect 4.3)

	//get platform ID

	//get a reference to the first available GPU device

	//Create a new OpenCL context on the selected device

	//Finally, create the command queue. All the asynchronous commands to the device will be issued
	//from the CPU into this queue. This way the host program can continue the execution until some results
	//from that device are needed.

	//////////////////////////////////////////////////////

	return false;
}

void CleanupContextResources()
{
	//////////////////////////////////////////////////////
	//(Sect 4.3)

	//release command queue and context
}

bool RunAssigment(IAssignment& pAssignment, size_t LocalWorkSize[3])
{
	if(!pAssignment.InitResources(g_CLDevice, g_CLContext))
	{
		cout<<"Error during resource allocation. Aborting execution."<<endl;
		pAssignment.ReleaseResources();
		return false;
	}

	//compute reference result
	cout<<"Computing CPU reference result...";
	pAssignment.ComputeCPU();
	cout<<"DONE"<<endl;

	//running the same task on the GPU
	cout<<"Computing GPU result...";
	
	pAssignment.ComputeGPU(g_CLContext, g_CLCommandQueue, LocalWorkSize);
	cout<<"DONE"<<endl;

	//validate results
	if(pAssignment.ValidateResults())
		cout<<"GOLD TEST PASSED!"<<endl;
	else
		cout<<"INVALID RESULTS!"<<endl;

	//cleanup
	pAssignment.ReleaseResources();

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////


